﻿using UnityEngine;
using System.Collections;

public class Ghost : MonoBehaviour {
	public float moveSpeed;	
	public StageTile currentTile;
	public StageTile goalTile;
	public int range;
	public int covered;
	public int distance;
	public StageHandler stageHandler;
	public bool detect;
	public bool feel;
	public float detectRange;
	public float detectMultiplier;
	public Character targetChar;
	public bool kill;

	private tk2dSprite sprite;

	public const float MOVE_TOLERANCE = 0.0001f;

	void Awake() {
		this.sprite = GetComponent<tk2dSprite>();
	}

	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
		if(this.kill) {
			return;
		}

		if(!this.feel) {
			if(Vector2.Distance((Vector2)this.stageHandler.char1.transform.position,
				(Vector2)this.transform.position) <= this.detectRange * this.detectMultiplier ||
				Vector2.Distance((Vector2)this.stageHandler.char2.transform.position,
 				(Vector2)this.transform.position) <= this.detectRange * this.detectMultiplier) {
					this.feel = true;
					this.stageHandler.char2.AddDetect();
			}
		}
		else {
			if(Vector2.Distance((Vector2)this.stageHandler.char1.transform.position,
            	(Vector2)this.transform.position) > this.detectRange * 2.0f &&
				Vector2.Distance((Vector2)this.stageHandler.char2.transform.position,
 				(Vector2)this.transform.position) > this.detectRange * 2.0f) {
				this.feel = false;
				this.stageHandler.char2.SubDetect();
			}
		}

		if(Vector2.Distance((Vector2)this.stageHandler.char1.transform.position,
    		(Vector2)this.transform.position) <= this.detectRange) {
			if(this.targetChar != this.stageHandler.char1) {
				this.targetChar = this.stageHandler.char1;
				this.detect = true;
			}
		}
		else if(Vector2.Distance((Vector2)this.stageHandler.char2.transform.position,
 			(Vector2)this.transform.position) <= this.detectRange) {
			if(this.targetChar != this.stageHandler.char2) {
				this.targetChar = this.stageHandler.char2;
				this.detect = true;
			}
		}
		else {
			this.detect = false;
			this.targetChar = null;
		}

		if(detect) {
			Stalk();
		}
		else {
			Patrol();
		}
	}

	private void Stalk() {
		Vector3 charPos = this.targetChar.transform.position;
		Vector3 ghostPos = this.transform.position;
		this.transform.position = Vector3.MoveTowards(ghostPos, new Vector3(charPos.x, charPos.y, ghostPos.z), Time.deltaTime * this.moveSpeed);
	
		if(charPos.x < ghostPos.x) {
			this.sprite.FlipX =	false;
		}
		else {
			this.sprite.FlipX = true;
		}
	}

	private void Patrol() {
		if(this.covered == this.range) {
			this.covered = 0;
			this.distance = this.distance == 1 ? -1 : 1;
		}

		if(this.distance == 1) {
			this.sprite.FlipX = true;
		}
		else {
			this.sprite.FlipX = false;
		}
		
		if(!this.goalTile) {
			this.goalTile = this.stageHandler.GetTile(this.currentTile.row, this.currentTile.col + this.distance);
			
			if(!this.goalTile || this.goalTile.locked) {
				this.distance = this.distance == 1 ? -1 : 1;
			}
		}
		else {
			Vector3 ghostPos = this.transform.position;
			Vector3 goalPos = this.goalTile.transform.position;
			this.transform.position = Vector3.MoveTowards(ghostPos, new Vector3(goalPos.x, goalPos.y, ghostPos.z), Time.deltaTime * this.moveSpeed);
			
			if(Vector2.Distance((Vector2)ghostPos, (Vector2)goalPos) < MOVE_TOLERANCE) {
				this.currentTile = this.goalTile;
				this.goalTile = null;
				this.transform.position = new Vector3(goalPos.x, goalPos.y, ghostPos.z);
				this.covered++;
			}
		}
	}

	void OnTriggerEnter(Collider other) {
		if(StageHandler.home) {
			return;
		}

		Character character = other.gameObject.GetComponent<Character>();
		StageHandler.PlayClip((AudioClip)Resources.Load("Audio/monster hit"));

		if(character) {
			this.stageHandler.char1.dead = true;
			this.stageHandler.char2.dead = true;
			this.kill = true;
			iTween.CameraFadeAdd();
			Hashtable ht = new Hashtable();
			ht.Add("amount", 1.0f);
			ht.Add("time", 3.0f);
			ht.Add("oncomplete", "ResetScene");
			ht.Add("oncompletetarget", this.gameObject);
			iTween.CameraFadeTo(ht);
		}
	}

	public void ResetScene() {
		Application.LoadLevel(this.stageHandler.currentScene);
	}

	void OnDrawGizmosSelected() {
		Gizmos.color = Color.red;
		Gizmos.DrawWireSphere(this.transform.position, this.detectRange);
	}
}
