﻿using UnityEngine;
using System.Collections;

public class StageHandler : MonoBehaviour {
	public StageTile[][] tileList;
	public GameObject tilePrefab;
	public GameObject charPrefab1;
	public GameObject charPrefab2;
	public StageTile startTile;
	public int rowNum;
	public int colNum;
	public Character char1;
	public Character char2;
	public string currentScene;

	public const float TILE_INTERVAL_X = 1.6f;
	public const float TILE_INTERVAL_Y = 1.6f;
	public const float START_X = 0.0f;
	public const float START_Y = 0.0f;
	public const float START_Z = 0.0f;
	public const float CHAR_ZPOS = -1.0f;

	public static AudioSource globalPlayer;
	public static bool home;

	void Awake() {
		StageHandler.home = false;
		InitTiles();
		InitCharacters();
		this.char1.SetViewActive(true);
		this.char2.SetViewActive(false);
		StageHandler.globalPlayer = GetComponent<AudioSource>();
	}

	// Use this for initialization
	void Start () {

	}
	
	// Update is called once per frame
	void Update () {
		if(Input.GetButtonDown("Jump")) {
			this.char1.goalTile = null;
			this.char2.goalTile = null;

			if(this.char1.activeControl) {
				this.char2.follow = false;
				this.char2.SetViewActive(true);
				this.char1.follow = true;
				this.char1.SetViewActive(false);
			}
			else {
				this.char1.follow = false;
				this.char1.SetViewActive(true);
				this.char2.follow = true;
				this.char2.SetViewActive(false);
			}

			StageHandler.PlayClip((AudioClip)Resources.Load("Audio/swap player"));
		}
	}

	public void InitTiles() {
		this.tileList = new StageTile[this.rowNum][];

		for(int i = 0; i < this.rowNum; i++) {
			this.tileList[i] = new StageTile[this.colNum];

			for(int j = 0; j < this.colNum; j++) {
				this.tileList[i][j] = GameObject.Find("Tile_" + i.ToString() + "_" + j.ToString()).GetComponent<StageTile>();
			}
		}
	}

	public void CreateTiles() {
		GameObject tileParent = new GameObject();
		tileParent.name = "Tiles";
		tileParent.transform.parent = this.transform;

		for(int i = 0; i < this.rowNum; i++) {
			float y = START_Y - (TILE_INTERVAL_Y * i);
			
			for(int j = 0; j < this.colNum; j++) {
				float x = START_X + (TILE_INTERVAL_X * j);
				GameObject tileObj = Instantiate(this.tilePrefab, new Vector3(x, y, START_Z), Quaternion.identity) as GameObject;
				tileObj.name = "Tile_" + i.ToString() + "_" + j.ToString();
				StageTile tile = tileObj.GetComponent<StageTile>();
				tile.row = i;
				tile.col = j;
				tileObj.transform.parent = tileParent.transform;
			}
		}
	}

	private void InitCharacters() {
		Vector3 tilePos = this.startTile.transform.position;
		GameObject charObj = GameObject.Instantiate(this.charPrefab1, new Vector3(tilePos.x, tilePos.y, CHAR_ZPOS), Quaternion.identity) as GameObject;
		Character character = charObj.GetComponent<Character>();
		character.currentTile = this.startTile;
		character.stageHandler = this;
		this.char1 = character;

		charObj = GameObject.Instantiate(this.charPrefab2, new Vector3(tilePos.x, tilePos.y, CHAR_ZPOS), Quaternion.identity) as GameObject;
		character = charObj.GetComponent<Character>();
		character.currentTile = this.startTile;
		character.stageHandler = this;
		this.char2 = character;

		this.char1.partner = this.char2;
		this.char2.partner = this.char1;
	}

	public StageTile GetTile(int row, int col) {
		if(row < 0 || row == this.rowNum || col < 0 || col == this.colNum) {
			return null;
		}

		return this.tileList[row][col];
	}

	public static void PlayClip(AudioClip clip) {
		globalPlayer.PlayOneShot(clip);
	}
}
