﻿using UnityEngine;
using System.Collections;

public class TimeManager : MonoBehaviour {
	public float nightSatValue;
	public float duration;
	public Material[] matSampler;
	public AudioClip dayClip;
	public AudioClip nightClip;
	public AudioSource ambientPlayer;

	private bool initialized;
	private bool fade;
	private bool night;
	private float startTime;
	private StageHandler stageHandler;

	void Awake() {
		this.stageHandler = GameObject.Find("StageHandler").GetComponent<StageHandler>();
		this.ambientPlayer.clip = this.dayClip;
		this.ambientPlayer.Play();
	}

	// Use this for initialization
	void Start () {
		this.startTime = Time.time;
		this.initialized = true;
	}
	
	// Update is called once per frame
	void Update () {
		if(this.initialized) {
			float percentage = (Time.time - this.startTime) / this.duration;
			float satValue = 1.0f - ((1.0f - this.nightSatValue) * percentage);

			foreach(Material mat in this.matSampler) {
				mat.SetFloat("_Sat", satValue);
			}

			if(satValue <= this.nightSatValue) {
				this.initialized = false;
				CameraFade();
			}

			if(percentage >= 0.5 && !this.night) {
				this.night = true;
				this.ambientPlayer.Stop();
				this.ambientPlayer.clip = this.nightClip;
				this.ambientPlayer.Play();
			}
		}
	}

	public void ResetMaterials() {
		foreach(Material mat in this.matSampler) {
			mat.SetFloat("_Sat", 1.0f);
		}
	}

	public void ResetScene() {
		Application.LoadLevel(this.stageHandler.currentScene);
	}

	private void CameraFade() {
		iTween.CameraFadeAdd();
		Hashtable ht = new Hashtable();
		ht.Add("amount", 1.0f);
		ht.Add("time", 5.0f);
		ht.Add("oncomplete", "ResetScene");
		ht.Add("oncompletetarget", this.gameObject);
		ht.Add("easetype", "easeInCirc");
		iTween.CameraFadeTo(ht);
	}
}
